//Datei: Toni-Futter.c
//
// Bedienung:
// Taster drcken startet das Gert
// Taster nochmal drcken schaltet in den Sleepmode
// 
// Copyright 2010 Matthias Kirchner
// ATtiny13 von 
// Compiler AVR-GCC
// Fuse-Bytes: high=0xFE, low=0x6A
// erst Programm flashen, dann FuseByte setzen! (128kHz 14CK+0ms, CLK/8, RSTDISBL)

// Standard-IO fr AVRs:
#include <avr/io.h>

// Bits wie Variablen benutzen:
#include "sbit.h"

// Fr AVRs gibt es eine Library fr sleep():
#include <avr/sleep.h>
// dann sind auch Interrupts notwendig:
#include <avr/interrupt.h>

enum { NIX, BLINK, STILL };

#define ONTIME 		2 		// Wartezeit EIN, 1 entspricht ca. 100ms
#define OFFTIME 	10 		// Wartezeit AUS, 1 entspricht ca. 100ms

#define NOP			asm volatile ("nop")

//Portnamen
#define LEDDIR		DDRB
#define LEDPORT		PORTB
#define INPUTPORT	PINB

#define L1			SBIT(LEDPORT,3)
#define L2			SBIT(LEDPORT,4)
#define TAST		PB5

#define OFF			1
#define ON			0

unsigned char mode;

void sleep_own(void)
{
 sei();
 sleep_enable();
 PCMSK = 1<<PCINT5;
 GIMSK = 1<<PCIE;
 sleep_cpu();
 GIMSK = 0;
 cli();
}

ISR(PCINT0_vect) 
{
mode=NIX;
}

void Delay1ms(unsigned char time)  
// eigentlich 100ms
{
//Nur verzgern wenn keine Taste gedrckt ist !
if bit_is_set(INPUTPORT,TAST) 
  {
  unsigned char k,l;
    {
    for(k=0; k<time; k++)
      {
      for(l=0; l<112; l++) // Pfriemelfaktor
        {
        NOP; NOP; NOP; NOP; NOP; NOP; NOP; NOP; NOP; NOP; 
        }
      }
    }
  }
}

int main()
{

unsigned int k, dauer=0;

// Port initialisieren:
LEDDIR = 0x18; 			// PB5/RESET/PCINT5 und PB0 Eingang, Rest Ausgang
LEDPORT = 0x38; 		// alle LEDs aus (1) und PullUp an PB5 ein

// Analog Komparator ausschalten:
ACSR = 0x80;
ADCSRA = 0x00;

// PowerSaver erlauben:
set_sleep_mode(2); 
sei();

// los geht's
if(mode==STILL) mode=NIX;    	//Wakeup aufgetreten, dann ist Taste gedrckt!!
else mode=BLINK;                //Batteriewechsel, dann Start mit Blinklicht
 
 while(1) // Anfang Programm Endlosschleife
  {
  if bit_is_clear(INPUTPORT,TAST)
    {
     while bit_is_clear(INPUTPORT,TAST)  //warten bis Taste losgelassen wird
      {
       for(k=0; k<1000; k++) //Entprellen! 
        {                   
        NOP; NOP; NOP; NOP; NOP;
        }
      }
     mode++; //Nchster mode
    }

    
//Lauflicht
   if(mode==BLINK) 
    {
	 dauer++;
	 if (dauer > 999) mode++;
     L1=ON; Delay1ms(ONTIME);
	 L1=OFF; Delay1ms(OFFTIME);
     L2=ON; Delay1ms(ONTIME);
	 L2=OFF; Delay1ms(OFFTIME);
    }


//Abschalten
if(mode==STILL) 
    {
	 dauer=0;
	 LEDPORT = 0x38;  //Alle Leds aus 
	 sleep_own(); // PowerDown, Warten auf nchsten Knopfdruck
    }     

  } // Ende Programm Endlosschleife
}


